@@ -23,13 +23,19 @@ import subprocess
 from pathlib import Path
 import apt_pkg
 from argparse import ArgumentParser
+import gettext
 
-from PyQt5.QtWidgets import (QWidget, QApplication, QLabel, QPushButton,
+from PyQt5.QtWidgets import (QWidget, QApplication, QLabel, QDialogButtonBox,
                              QHBoxLayout, QVBoxLayout, QTreeWidget,
-                             QTreeWidgetItem)
+                             QTreeWidgetItem, QHeaderView)
 from PyQt5.QtCore import Qt
 from PyQt5.QtGui import QIcon
+import importlib.util
 
+spec = importlib.util.spec_from_file_location(
+    "apt_check", "/usr/lib/update-notifier/apt_check.py")
+apt_check = importlib.util.module_from_spec(spec)
+spec.loader.exec_module(apt_check)
 
 class Dialog(QWidget):
     ''' UI '''
@@ -45,13 +51,14 @@ class Dialog(QWidget):
         try:
             self.cache = apt_pkg.Cache()
         except SystemError as e:
-            sys.stderr.write("Error: Opening the cache (%s)" % e)
+            sys.stderr.write(_("Error: Opening the cache (%s)") % e)
             sys.exit(-1)
         self.depcache = apt_pkg.DepCache(self.cache)
+        self.records = apt_pkg.PackageRecords(self.cache)
 
         self.initUI()
-        self.upgradeBtn.clicked.connect(self.call_upgrade)
-        self.closeBtn.clicked.connect(self.call_reject)
+        self.buttonBox.rejected.connect(self.call_reject)
+        self.buttonBox.clicked.connect(self.call_upgrade)
 
     def initUI(self):
         ''' UI initialization '''
@@ -59,18 +66,24 @@ class Dialog(QWidget):
         self.label.setAlignment(Qt.AlignHCenter)
 
         self.tw = QTreeWidget()
-        self.tw.setColumnCount(1)
-        self.tw.setHeaderLabels(['Affected Packages'])
-        self.tw.setHeaderHidden(True)
+        if self.security_upgrades > 0:
+            self.tw.setColumnCount(2)
+            self.tw.setHeaderLabels([_('Affected Packages'),
+                                     _('Security')])
+            self.tw.header().setSectionResizeMode(0, QHeaderView.Stretch)
+            self.tw.header().setStretchLastSection(False)
+        else:
+            self.tw.setColumnCount(1)
+            self.tw.setHeaderLabels([_('Affected Packages')])
+            self.tw.setHeaderHidden(True)
 
-        self.upgradeBtn = QPushButton("Upgrade")
-        self.closeBtn = QPushButton("Close")
+        self.buttonBox = QDialogButtonBox(QDialogButtonBox.Cancel |
+                                          QDialogButtonBox.Apply)
         text = ""
 
         hbox = QHBoxLayout()
         hbox.addStretch(1)
-        hbox.addWidget(self.upgradeBtn)
-        hbox.addWidget(self.closeBtn)
+        hbox.addWidget(self.buttonBox)
         hbox.addStretch(1)
 
         vbox = QVBoxLayout()
@@ -78,8 +91,10 @@ class Dialog(QWidget):
         vbox.addWidget(self.tw)
         vbox.addLayout(hbox)
 
+        self.tw.setVisible(False)
+
         if self.upg_path is None:
-            self.upgradeBtn.setVisible(False)
+            self.buttonBox.button(QDialogButtonBox.Apply).setVisible(False)
 
         self.setLayout(vbox)
         self.setGeometry(300, 300, 500, 150)
@@ -87,49 +102,75 @@ class Dialog(QWidget):
         self.center()
 
         if self.upgrades > 0:
+            self.tw.setVisible(True)
             self.depcache.upgrade(True)  # True for non safe.
             pkg_install = list()
             pkg_upgrade = list()
             pkg_delete = list()
             for p in self.cache.packages:
                 if self.depcache.marked_delete(p):
-                    pkg_delete.append(p.name)
+                    pkg_delete.append(p)
                 elif self.depcache.marked_install(p):
-                    pkg_install.append(p.name)
+                    pkg_install.append([p, self.depcache.get_candidate_ver(p)])
                 elif self.depcache.marked_upgrade(p):
-                    pkg_upgrade.append(p.name)
-            text = "There are upgrades available. Do you want to do a system"
-            text += " upgrade?\nThis will mean packages could be upgraded,"
-            text += " installed, or removed."
+                    pkg_upgrade.append([p, self.depcache.get_candidate_ver(p)])
+            text = _("There are upgrades available. Do you want to do a system"
+                     " upgrade?")
+            text += "\n"
+            text += _("This will mean packages could be upgraded, installed or"
+                      " removed.")
+            if self.security_upgrades > 0:
+                text += "\n" + str(self.security_upgrades)
+                if self.security_upgrades == 1:
+                    text += _(" is a security upgrade.")
+                else:
+                    text += _(" are security upgrades.")
 
             if len(pkg_delete) > 0:
-                toDelete = QTreeWidgetItem(['Remove'])
+                toDelete = QTreeWidgetItem([_('Remove')])
                 for p in pkg_delete:
-                    td_child = QTreeWidgetItem([p])
+                    td_child = QTreeWidgetItem(p.name)
                     toDelete.addChild(td_child)
                 toDelete.setIcon(0, QIcon.fromTheme("edit-delete"))
                 self.tw.addTopLevelItem(toDelete)
             if len(pkg_install) > 0:
-                toInstall = QTreeWidgetItem(['Install'])
+                toInstall = QTreeWidgetItem([_('Install')])
                 for p in pkg_install:
-                    td_child = QTreeWidgetItem([p])
+                    td_child = QTreeWidgetItem([p[0].name + "  /  "
+                                                + p[1].ver_str])
+                    if apt_check.isSecurityUpgrade(p[1]):
+                        td_child.setIcon(1, QIcon.fromTheme("security-high"))
+                        toInstall.setIcon(1, QIcon.fromTheme("security-high"))
                     toInstall.addChild(td_child)
-                toInstall.setIcon(0, QIcon.fromTheme("system-software-install"))
+                    self.records.lookup(p[1].file_list[0])
+                    short = QTreeWidgetItem([self.records.short_desc])
+                    td_child.addChild(short)
+                toInstall.setIcon(0, QIcon.fromTheme(
+                    "system-software-install"))
                 self.tw.addTopLevelItem(toInstall)
             if len(pkg_upgrade) > 0:
-                toUpgrade = QTreeWidgetItem(['Upgrade'])
+                toUpgrade = QTreeWidgetItem([_('Upgrade')])
                 for p in pkg_upgrade:
-                    td_child = QTreeWidgetItem([p])
+                    td_child = QTreeWidgetItem(
+                        [p[0].name + "  /  " + p[0].current_ver.ver_str +
+                         "  ->  " + p[1].ver_str])
+                    if apt_check.isSecurityUpgrade(p[1]):
+                        td_child.setIcon(1, QIcon.fromTheme("security-high"))
+                        toUpgrade.setIcon(1, QIcon.fromTheme("security-high"))
                     toUpgrade.addChild(td_child)
+                    self.records.lookup(p[1].file_list[0])
+                    short = QTreeWidgetItem([self.records.short_desc])
+                    td_child.addChild(short)
                 toUpgrade.setIcon(0, QIcon.fromTheme("system-software-update"))
                 self.tw.addTopLevelItem(toUpgrade)
 
         if self.reboot_required:
             if text == "":
-                text = "Reboot is needed"
-                self.upgradeBtn.setVisible(False)
+                text = _("Reboot required")
+                self.buttonBox.button(QDialogButtonBox.Apply).setVisible(False)
             else:
-                text = text + "\nReboot is needed"
+                text += "\n"
+                text += _("Reboot required")
 
         self.label.setText(text)
 
@@ -146,38 +187,36 @@ class Dialog(QWidget):
         ''' when close button is pressed, quit '''
         app.quit()
 
-    def call_upgrade(self):
-        ''' starts upgrade process '''
-        self.label.setText("Upgrading...")
-        # TODO maybe open another thread so notifier won't freeze
-        if self.upg_path == "terminal":
-            # cmd = ['qterminal', '-e', 'sudo', 'apt', 'dist-upgrade']
-            cmd = ['qterminal', '-e', './upg.sh']
-        else:
+    def call_upgrade(self, btnClicked):
+        if(self.buttonBox.buttonRole(btnClicked) ==
+           QDialogButtonBox.ApplyRole):
+            ''' starts upgrade process '''
+            self.label.setText(_("Upgrading..."))
+            # TODO maybe open another thread so notifier won't freeze
             cmd = ['lxqt-sudo', self.upg_path, '--full-upgrade']
-        # process = subprocess.Popen(self.upg_path)
-        # process = subprocess.Popen(cmd, shell=True)
-        self.upgradeBtn.setVisible(False)
-        self.upgradeBtn.setEnabled(False)
-        process = subprocess.Popen(cmd)
-        process.wait()
+            self.buttonBox.button(QDialogButtonBox.Apply).setEnabled(False)
+            self.buttonBox.button(QDialogButtonBox.Apply).setVisible(False)
+            self.tw.setVisible(False)
+            process = subprocess.Popen(cmd)
+            process.wait()
 
-        if self.upg_path == "terminal":
-            text = "Upgrade finished"
+            if self.upg_path == "terminal":
+                text = _("Upgrade finished")
 
-            reboot_required_path = Path("/var/run/reboot-required")
-            if reboot_required_path.exists():
-                text = text + "\n" + "Restart required"
-            self.label.setText(text)
-            self.closeBtn.setVisible(True)
-            self.closeBtn.setEnabled(True)
+                reboot_required_path = Path("/var/run/reboot-required")
+                if reboot_required_path.exists():
+                    text += "\n" + _("Reboot required")
+                self.label.setText(text)
+                self.closeBtn.setVisible(True)
+                self.closeBtn.setEnabled(True)
 
-        else:
-            app.quit()
+            else:
+                app.quit()
 
 
 class App(QApplication):
     '''application'''
+
     def __init__(self, upgrades, security_upgrades, reboot_required, upg_path,
                  *args):
         QApplication.__init__(self, *args)
@@ -195,21 +234,27 @@ def main(args, upgrades, security_upgrades, reboot_required, upg_path):
 
 
 if __name__ == "__main__":
+    localesApp = "lubuntu-update-notifier"
+    localesDir = "/usr/share/locale"
+    gettext.bindtextdomain(localesApp, localesDir)
+    gettext.textdomain(localesApp)
+    _ = gettext.gettext
+
     parser = ArgumentParser()
     parser.add_argument("-p",
                         "--upgrader-sw",
                         dest="upg_path",
-                        help="Define software/app to open for upgrade",
+                        help=_("Define software/app to open for upgrade"),
                         metavar="APP")
     parser.add_argument("-u",
                         "--upgrades",
                         dest="upgrades",
-                        help="How many upgrades are available",
+                        help=_("How many upgrades are available"),
                         metavar="APP")
     parser.add_argument("-s",
                         "--security-upg",
                         dest="security_upgrades",
-                        help="How many security upgrades are available",
+                        help=_("How many security upgrades are available"),
                         metavar="APP")
 
     options = parser.parse_args()
